package ru.dbotthepony.kstarbound.client.network.packets

import ru.dbotthepony.kstarbound.client.network.ClientConnection
import ru.dbotthepony.kstarbound.network.IClientPacket
import ru.dbotthepony.kstarbound.util.readChunkPos
import ru.dbotthepony.kstarbound.util.readCollection
import ru.dbotthepony.kstarbound.util.writeCollection
import ru.dbotthepony.kstarbound.util.writeVec2i
import ru.dbotthepony.kstarbound.world.CHUNK_SIZE
import ru.dbotthepony.kstarbound.world.Chunk
import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kstarbound.world.api.ImmutableCell
import ru.dbotthepony.kstarbound.world.api.MutableCell
import java.io.DataInputStream
import java.io.DataOutputStream

class InitialChunkDataPacket(val pos: ChunkPos, val data: List<ImmutableCell>) : IClientPacket {
	constructor(stream: DataInputStream) : this(stream.readChunkPos(), stream.readCollection { MutableCell().read(stream).immutable() })
	constructor(chunk: Chunk<*, *>) : this(chunk.pos, ArrayList<ImmutableCell>(CHUNK_SIZE * CHUNK_SIZE).also {
		for (x in 0 until CHUNK_SIZE) {
			for (y in 0 until CHUNK_SIZE) {
				it.add(chunk.getCell(x, y).immutable())
			}
		}
	})

	override fun write(stream: DataOutputStream) {
		stream.writeVec2i(pos)
		stream.writeCollection(data) { it.write(stream) }
	}

	override fun play(connection: ClientConnection) {
		val chunk = connection.client.world?.chunkMap?.compute(pos.x, pos.y) ?: return
		val itr = data.iterator()

		for (x in 0 until CHUNK_SIZE) {
			for (y in 0 until CHUNK_SIZE) {
				chunk.setCell(x, y, itr.next())
			}
		}
	}
}
